@@ -1,22 +1,33 @@
+"""
+Module for manage dataset using local credentials and config files
+"""
+# pylint: disable=line-too-long, invalid-name, too-many-arguments, invalid-envvar-value,line-too-long
+from pathlib import Path
+import sys
+from os import getenv
+import shutil
+import warnings
+import base64
+import json
+from functools import lru_cache
+
 from google.cloud import bigquery, storage
 from google.oauth2 import service_account
+from loguru import logger
 import yaml
 from jinja2 import Template
-from pathlib import Path
-import shutil
 import tomlkit
-import warnings
-import json
-import base64
-from os import getenv
-from basedosdados import constants
 
-from functools import lru_cache
+from basedosdados.constants import config, constants
 
 warnings.filterwarnings("ignore")
 
 
 class Base:
+    """
+    Base class for all datasets
+    """
+
     def __init__(
         self,
         config_path=".basedosdados",
@@ -25,40 +36,59 @@ class Base:
         metadata_path=None,
         overwrite_cli_config=False,
     ):
+        """
+        Initialize the class
+        """
+        # standard config_path configuration
+        config_path = (
+            Path(config.project_config_path)
+            if config.project_config_path is not None
+            else Path.home() / config_path
+        )
 
-        self.config_path = Path.home() / config_path
+        self.config_path =  config_path
         self._init_config(force=overwrite_cli_config)
         self.config = self._load_config()
+        self._config_log(config.verbose)
 
         self.templates = Path(templates or self.config["templates_path"])
-        self.metadata_path = Path(
-            metadata_path or self.config["metadata_path"]
-        )
+        self.metadata_path = Path(metadata_path or self.config["metadata_path"])
         self.bucket_name = bucket_name or self.config["bucket_name"]
         self.uri = f"gs://{self.bucket_name}" + "/staging/{dataset}/{table}/*"
 
     @staticmethod
     def _decode_env(env: str) -> str:
-        return base64.b64decode(getenv(env).encode('utf-8')).decode('utf-8')
+        """
+        Decode environment variable
+        """
+        return base64.b64decode(getenv(env).encode("utf-8")).decode("utf-8")
 
     def _load_credentials(self, mode: str):
+        """
+        Load credentials from config file
+        """
 
         if getenv(f"{constants.ENV_CREDENTIALS_PREFIX.value}{mode.upper()}"):
-            info = json.loads(self._decode_env(
-                f"{constants.ENV_CREDENTIALS_PREFIX.value}{mode.upper()}"))
+            info = json.loads(
+                self._decode_env(
+                    f"{constants.ENV_CREDENTIALS_PREFIX.value}{mode.upper()}"
+                )
+            )
             return service_account.Credentials.from_service_account_info(
-                info, scopes=[
-                    "https://www.googleapis.com/auth/cloud-platform"]
-            )
-        else:
-            return service_account.Credentials.from_service_account_file(
-                self.config["gcloud-projects"][mode]["credentials_path"],
-                scopes=["https://www.googleapis.com/auth/cloud-platform"],
+                info, scopes=["https://www.googleapis.com/auth/cloud-platform"]
             )
 
+        return service_account.Credentials.from_service_account_file(
+            self.config["gcloud-projects"][mode]["credentials_path"],
+            scopes=["https://www.googleapis.com/auth/cloud-platform"],
+        )
+
     @property
     @lru_cache(256)
     def client(self):
+        """
+        Client for BigQuery
+        """
 
         return dict(
             bigquery_prod=bigquery.Client(
@@ -77,7 +107,9 @@ class Base:
 
     @property
     def main_vars(self):
-
+        """
+        Variables for main templates
+        """
         return dict(
             templates=self.templates,
             metadata_path=self.metadata_path,
@@ -86,15 +118,19 @@ class Base:
 
     @staticmethod
     def _input_validator(context, default="", with_lower=True):
+        """
+        Validate input
+        """
 
         var = input(context)
 
-        def to_lower(x): return x.lower() if with_lower else x
+        def to_lower(x):
+            return x.lower() if with_lower else x
 
         if var:
             return to_lower(var.strip())
-        else:
-            return to_lower(default.strip())
+
+        return to_lower(default.strip())
 
     def _selection_yn(
         self,
@@ -105,6 +141,9 @@ class Base:
         default_no="",
         with_lower=True,
     ):
+        """
+        Selection with yes/no
+        """
 
         while True:
 
@@ -112,18 +151,22 @@ class Base:
 
             if res == "y":
                 return default_return
-            elif res == "n":
-                return self._input_validator(no_question, default_no, with_lower)
-            else:
-                print(f"{res} is not accepted as an awnser. Try y or n.\n")
 
+            if res == "n":
+                return self._input_validator(no_question, default_no, with_lower)
+
+            print(f"{res} is not accepted as an awnser. Try y or n.\n")
+
+    @staticmethod
     def _check_credentials(
-        self,
         project_id,
         mode,
         credentials_path,
         credentials_url="https://console.cloud.google.com/apis/credentials/serviceaccountkey?project=",
     ):
+        """
+        Check if credentials are valid
+        """
 
         input(
             f"1. Go to the following link: {credentials_url}{project_id}\n"
@@ -139,14 +182,16 @@ class Base:
             if (credentials_path / f"{mode}.json").exists():
                 print("We found it :)")
                 break
-            else:
-                input(
-                    "\nWe couldn't find it :( \nMake sure that the file has the following name:"
-                    f"\n\n{mode} filename: {credentials_path}/{mode}.json"
-                    "\n\n[Press enter to check if data is correctly saved]"
-                )
+            input(
+                "\nWe couldn't find it :( \nMake sure that the file has the following name:"
+                f"\n\n{mode} filename: {credentials_path}/{mode}.json"
+                "\n\n[Press enter to check if data is correctly saved]"
+            )
 
     def _init_config(self, force):
+        """
+        Initialize config file
+        """
 
         # Create config folder
         self.config_path.mkdir(exist_ok=True, parents=True)
@@ -154,7 +199,7 @@ class Base:
         config_file = self.config_path / "config.toml"
 
         # Create credentials folder
-        credentials_folder = Path.home() / self.config_path / "credentials"
+        credentials_folder = self.config_path / "credentials"
         credentials_folder.mkdir(exist_ok=True, parents=True)
 
         # Create template folder
@@ -162,30 +207,27 @@ class Base:
 
         # If environments are set but no files exist
         if (
-            (not config_file.exists()) and
-            (getenv(constants.ENV_CONFIG.value)) and
-            (getenv(constants.ENV_CREDENTIALS_PROD.value)) and
-            (getenv(constants.ENV_CREDENTIALS_STAGING.value))
+            (not config_file.exists())
+            and (getenv(constants.ENV_CONFIG.value))
+            and (getenv(constants.ENV_CREDENTIALS_PROD.value))
+            and (getenv(constants.ENV_CREDENTIALS_STAGING.value))
         ):
             # Create basedosdados files from envs
-            with open(config_file, "w") as f:
+            with open(config_file, "w", encoding="utf-8") as f:
                 f.write(self._decode_env(constants.ENV_CONFIG.value))
                 f.close()
-            with open(credentials_folder / "prod.json", "w") as f:
+            with open(credentials_folder / "prod.json", "w", encoding="utf-8") as f:
                 f.write(self._decode_env(constants.ENV_CREDENTIALS_PROD.value))
                 f.close()
-            with open(credentials_folder / "staging.json", "w") as f:
-                f.write(self._decode_env(
-                    constants.ENV_CREDENTIALS_STAGING.value))
+            with open(credentials_folder / "staging.json", "w", encoding="utf-8") as f:
+                f.write(self._decode_env(constants.ENV_CREDENTIALS_STAGING.value))
                 f.close()
 
         if (not config_file.exists()) or (force):
 
             # Load config file
             c_file = tomlkit.parse(
-                (Path(__file__)
-                    .resolve()
-                    .parents[1] / "configs" / "config.toml")
+                (Path(__file__).resolve().parents[1] / "configs" / "config.toml")
                 .open("r", encoding="utf-8")
                 .read()
             )
@@ -209,9 +251,7 @@ class Base:
                 ),
                 default_yn="y",
                 default_return=Path.cwd(),
-                no_question=(
-                    "\nWhere would you like to save it?\n" "metadata path: "
-                ),
+                no_question=("\nWhere would you like to save it?\n" "metadata path: "),
                 with_lower=False,
             )
 
@@ -219,7 +259,7 @@ class Base:
 
             ############# STEP 2 - CREDENTIALS PATH ######################
 
-            credentials_path = Path.home() / ".basedosdados" / "credentials"
+            credentials_path = self.config_path / "credentials"
             credentials_path = Path(
                 self._selection_yn(
                     first_question=(
@@ -246,9 +286,7 @@ class Base:
                 "basedosdados-staging",
             )
 
-            self._check_credentials(
-                project_staging, "staging", credentials_path
-            )
+            self._check_credentials(project_staging, "staging", credentials_path)
 
             c_file["gcloud-projects"]["staging"]["credentials_path"] = str(
                 credentials_path / "staging.json"
@@ -297,26 +335,41 @@ class Base:
 
             ############# STEP 6 - SET TEMPLATES #######################
 
-            c_file["templates_path"] = str(
-                Path.home() / ".basedosdados" / "templates"
-            )
+            c_file["templates_path"] = str(self.config_path / "templates")
 
-            config_file.open(
-                "w", encoding="utf-8").write(tomlkit.dumps(c_file)
-            )
+            config_file.open("w", encoding="utf-8").write(tomlkit.dumps(c_file))
+
+    @staticmethod
+    def _config_log(verbose: bool):
+        """
+        Logging configuration
+        """
+        logger.remove()  # remove o default handler
+        logger_level = "INFO" if verbose else "ERROR"
+        logger.add(
+            sys.stderr,
+            level=logger_level,
+            format="<green>{time:YYYY-MM-DD HH:mm:ss.SSS}</green> | <level>{level: <8}</level> | <cyan>{name}</cyan>:<cyan>{function}</cyan>:<cyan>{line}</cyan> - <level>{message}</level>",
+        )
 
     def _load_config(self):
+        """
+        Loads the configuration file
+        """
 
         if getenv(constants.ENV_CONFIG.value):
             return tomlkit.parse(self._decode_env(constants.ENV_CONFIG.value))
-        else:
-            return tomlkit.parse(
-                (Path.home() / ".basedosdados" / "config.toml")
-                .open("r", encoding="utf-8")
-                .read()
-            )
+        return tomlkit.parse(
+            (self.config_path / "config.toml")
+            .open("r", encoding="utf-8")
+            .read()
+        )
 
-    def _load_yaml(self, file):
+    @staticmethod
+    def _load_yaml(file):
+        """
+        Loads a yaml file
+        """
 
         try:
             return yaml.load(open(file, "r", encoding="utf-8"), Loader=yaml.SafeLoader)
@@ -324,23 +377,41 @@ class Base:
             return None
 
     def _render_template(self, template_file, kargs):
+        """
+        Render a template file
+        """
 
         return Template(
             (self.templates / template_file).open("r", encoding="utf-8").read()
         ).render(**kargs)
 
-    def _check_mode(self, mode):
-        ACCEPTED_MODES = ["all", "staging", "prod", "raw"]
+    @staticmethod
+    def _check_mode(mode):
+        """
+        Checks if the mode is valid
+        """
+        ACCEPTED_MODES = [
+            "all",
+            "staging",
+            "prod",
+            "raw",
+            "header",
+            "auxiliary_files",
+            "architecture",
+        ]
         if mode in ACCEPTED_MODES:
             return True
-        else:
-            raise Exception(
-                f"Argument {mode} not supported. "
-                f"Enter one of the following: "
-                f'{",".join(ACCEPTED_MODES)}'
-            )
+
+        raise Exception(
+            f"Argument {mode} not supported. "
+            f"Enter one of the following: "
+            f'{",".join(ACCEPTED_MODES)}'
+        )
 
     def _refresh_templates(self):
+        """
+        Refreshes the templates
+        """
         shutil.rmtree((self.config_path / "templates"), ignore_errors=True)
         shutil.copytree(
             (Path(__file__).resolve().parents[1] / "configs" / "templates"),
